package cz.vutbr.fit.gja.startdraw;

import android.Manifest;
import android.app.Activity;
import android.app.AlertDialog;
import android.app.Dialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.database.Cursor;
import android.graphics.Color;
import android.net.Uri;
import android.os.Bundle;
import android.provider.MediaStore;
import androidx.core.app.ActivityCompat;
import android.view.Menu;
import android.view.MenuItem;

public class StartDraw extends Activity {
  DrawView drawView;

  @Override
  public void onCreate(Bundle savedInstanceState) {
    super.onCreate(savedInstanceState);

    final int REQUEST_CODE = 1;
    if (!(checkSelfPermission(
            android.Manifest.permission.READ_EXTERNAL_STORAGE) == PackageManager.PERMISSION_GRANTED)) {
      ActivityCompat.requestPermissions(this, new String[]{Manifest.permission.READ_EXTERNAL_STORAGE}, REQUEST_CODE);
    }

    drawView = new DrawView(this);
    drawView.setBackgroundColor(Color.WHITE);
    setContentView(drawView);
  }
  
  @Override
  public boolean onCreateOptionsMenu(Menu menu) {
    // Inflate the menu; this adds items to the action bar if it is present.
    getMenuInflater().inflate(R.menu.main, menu);
    return true;
  }
  
  public boolean onOptionsItemSelected(MenuItem item) {
    int id=item.getItemId();
    if (id == R.id.load) {
      loadImage();
      return true;
    } 
    else if (id == R.id.start)
      drawView.start();
    else if (id == R.id.next)
      drawView.next();
    else if (id == R.id.prev)
      drawView.prev();
    else if (id == R.id.show)
      drawView.clear();
    else if (id==R.id.grid)
      gridDialog();

    return (super.onOptionsItemSelected(item));
  }   
  
  public void gridDialog(){
    final CharSequence[] items = {"4","6","8"};
    
    // Creating and Building the Dialog 
    AlertDialog.Builder builder = new AlertDialog.Builder(this);
    builder.setTitle("Select grid size");
    builder.setSingleChoiceItems(items, -1, new DialogInterface.OnClickListener() {
    public void onClick(DialogInterface dialog, int item) {
      drawView.changeGrid(4+2*item);
      Dialog.class.cast(dialog).dismiss();   
      }
    });
    builder.create().show();
  }
  
  public void loadImage(){
    Intent i = new Intent(
        Intent.ACTION_PICK,
        android.provider.MediaStore.Images.Media.EXTERNAL_CONTENT_URI);

    startActivityForResult(i, 1);
  }
  
  @Override
  protected void onActivityResult(int requestCode, int resultCode, Intent data) {
    super.onActivityResult(requestCode, resultCode, data);

    if (requestCode == 1 && resultCode == RESULT_OK && null != data) {
      Uri selectedImage = data.getData();
      String[] filePathColumn = { MediaStore.Images.Media.DATA };

      Cursor cursor = getContentResolver().query(selectedImage,
          filePathColumn, null, null, null);
      cursor.moveToFirst();

      int columnIndex = cursor.getColumnIndex(filePathColumn[0]);
      String picturePath = cursor.getString(columnIndex);
      cursor.close();
      drawView.addImage(picturePath);
      
    }
   }
}